SystemOrganization addCategory: #'Magma seasideHelper-Helpers'!
SystemOrganization addCategory: #'Magma seasideHelper-ControlPanel'!
SystemOrganization addCategory: #'Magma seasideHelper-Model'!
SystemOrganization addCategory: #'Magma seasideHelper-Configuration'!
SystemOrganization addCategory: #'Magma seasideHelper-ReadStrategy'!
SystemOrganization addCategory: #'Magma seasideHelper-ReadStrategy-Tests'!

TestCaseUsingMagma subclass: #WAReadStrategyClassSpecTest
	instanceVariableNames: 'rs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ReadStrategy-Tests'!

!WAReadStrategyClassSpecTest methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 05:48'!
readStrategyClass

	^ WAMagmaRSDefault! !

!WAReadStrategyClassSpecTest methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 04:03'!
setUp

	self magma initializeRoot: Dictionary new.! !

!WAReadStrategyClassSpecTest methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 05:34'!
testSimpleCase

	self magma session commit: [ self magma session root at: 'TestCase' put: TestCase new ].

	rs := self readStrategyClass minimumDepth: 1.
 
	self magma session readStrategy: rs. 



	"

	self debug: #testSimpleCase


	"! !

MaReadStrategy subclass: #WAMagmaRSTraditional
	instanceVariableNames: 'session'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ReadStrategy'!

!WAMagmaRSTraditional commentStamp: 'kph 12/10/2007 08:13' prior: 0!
This is the basic traditional magma read strategy implementation. It internally uses the original terse data structure which is efficient but difficult to debug. 
Read strategies are built by the helper and are updated whenever there is a new seaside session.!

WAMagmaRSTraditional subclass: #WAMagmaRSDefault
	instanceVariableNames: 'fullSpecifications'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ReadStrategy'!

!WAMagmaRSDefault commentStamp: 'kph 12/10/2007 08:15' prior: 0!
This is the default magma seaside read strategy implementation. Internally it uses a verbose data structure which is easier to debug than the traditional one. 
Read strategies are built by the helper and are updated whenever there is a new seaside session. 
<br><br>
The following enhancements are included
<ol>
<li>Specify a default depth AND individual depths for particular instVars</li>
<li>Negative values are valid and are honoured</li>
<li>Additional, easier to use but less compatible, interface for building strategies</li>
</ol>
 
!

!WAMagmaRSDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 04:30'!
convertSpecificationsToIdsUsing: aMaClassIdManager 
	"Convert the classes in the receiver to classId's, since that is what the server goes by."
	| specifiedClassNames |
	specifiedClassNames _ depthSpecifications keys collect: [ : each | each name ].

	aMaClassIdManager classDefinitionsById do: 
		[ : eachDefinitions | 
		eachDefinitions do: 
			[ : eachDefinition | 
			"Check superclasses to inherit their specification, but only if one was not already specified for eachDefinition."
			eachDefinition withAllSuperclassDefinitionsDo: 
				[ : eachDef | 
				(specifiedClassNames includes: eachDef name) ifTrue: 
					[ depths 
						at: { eachDefinition id. eachDefinition version }
						ifAbsentPut: 
							[ self beNew.
							self instVarMapFrom: eachDef for: eachDefinition ] ] ] ] ]
	! !

!WAMagmaRSDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 01:18'!
depthArrayFor: aMaClassDefinition defaultDepth: defaultDepth


	"Given 

	MySuperclass with instvers: 'a b c'
	MySubclass with instvars: 'd e f'

	we turn onAny: MySubclass defaultDepth: 3

	into #(nil nil nil 3 3 3) "


	| result nInherited |

	result := Array new: aMaClassDefinition namedInstSize.

	nInherited := aMaClassDefinition superclassDefinition namedInstSize.

	nInherited to: result size do: [ :n |
	
		result at: n put: defaultDepth
	].

	^ defaultDepth! !

!WAMagmaRSDefault methodsFor: 'compatability' stamp: 'kph 6/19/2007 01:51'!
forVariableNamed: aString onAny: aClass readToDepth: anInteger

	"compatibility"

	(self onAny: aClass) readVar: aString toDepth: anInteger! !

!WAMagmaRSDefault methodsFor: 'compatability' stamp: 'kph 6/19/2007 01:51'!
forVariablesNamed: aString onAny: aClass readToDepth: anInteger

	"compatibility"

	(self onAny: aClass) readVars: aString toDepth: anInteger! !

!WAMagmaRSDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 06:42'!
inheritDepthsUsing: aMaClassIdManager 
	 

	depths do: [ :ea | ea inheritDepthsIn: depths using: aMaClassIdManager ].
			 
			! !

!WAMagmaRSDefault methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 04:31'!
instVarMapFrom: aMaClassDefinition for: aMaSubclassDefinition 
	"Answer an Array with an instVarMap of aMaClassDefinition according to user-declared read depths."
	| |

	(aMaClassDefinition isDefined and: [ depthSpecifications includesKey: aMaClassDefinition classObject ]) 
		ifFalse: [ ^ nil ].

	^ (depthSpecifications at: aMaClassDefinition classObject) copy makeInstVarMapFor: aMaSubclassDefinition! !

!WAMagmaRSDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 23:25'!
makeReadyForUseUsing: aMaClassIdManager

	self 
		resetDepths ;
		convertSpecificationsToIdsUsing: aMaClassIdManager.
	self inheritDepthsUsing: aMaClassIdManager.
	fullSpecifications := depths copy. "make a copy of the fully processed specs for debugging etc"
	
	self tidyUpDepths.! !

!WAMagmaRSDefault methodsFor: 'nil' stamp: 'kph 12/10/2007 06:46'!
onAny: aClass

	"the front end to the easier to use read strategy specificaiton interface"
	
	^ depthSpecifications 
		at: aClass
		ifAbsentPut: 
			[ WAReadStrategyClassSpec forClass: aClass ]! !

!WAMagmaRSDefault methodsFor: 'compatability' stamp: 'kph 6/19/2007 01:52'!
onAny: aClass readToDepth: anInteger 

	(self onAny: aClass) readAllVarsToDepth: anInteger! !

!WAMagmaRSDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 23:26'!
resetSpecifications

	depthSpecifications := nil.
	fullSpecifications := nil.! !

!WAMagmaRSDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 05:12'!
tidyUpDepths

	depths keys do: [ :key |
		depths at: key put: (self tidyUpDepths: (depths at: key))
	].! !

!WAMagmaRSDefault methodsFor: 'nil' stamp: 'kph 6/19/2007 06:43'!
tidyUpDepths: spec

	| a array |

	array := spec instVarMap.
	
	array isEmpty ifTrue: [ ^ 1 ].

	array detect: [ :ea | (a ifNil: [a := ea ])  ~= ea ] ifNone: [ ^ a ].

	^ array! !

WAMagmaRSDefault subclass: #WAMagmaRSLogging
	instanceVariableNames: 'firstTime readStart totalTime timeStamp readCount entryIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ReadStrategy'!

!WAMagmaRSLogging commentStamp: 'kph 12/10/2007 07:46' prior: 0!
This adds logging to the default magma seaside read strategy (using the Logging package)!

!WAMagmaRSLogging methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 07:08'!
logNoDecisions

	^ false! !

!WAMagmaRSLogging methodsFor: 'as yet unclassified' stamp: 'kph 6/13/2007 20:06'!
reset

	firstTime := nil.
	entryIndex := 0.! !

!WAMagmaRSLogging methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 06:59'!
signalBeginningToRead: oidInteger classId: classId

"context expected for
entryForReading: oid at: currentDepth to: minDepth attribute: varIndex ofClassId: classId version: classVersion 
"
	| theValue   theClass |

	readStart ifNotNil: [ totalTime := totalTime + timeStamp - readStart ].
					
	readStart := timeStamp := Time millisecondClockValue.

	firstTime ifNil: [ firstTime := readStart. readCount  := 0. totalTime := 0 ].
	readCount := readCount + 1.
		
	theValue := session serializer objectWithOid: oidInteger.

	theValue maIsMutatingProxy 
		ifTrue: [ 

			theClass := session serializer classIdManager classForId: classId ifAbsent: [ ^ self error: 'could not find class id' ].

			^ self log magma readCount: readCount totalMs: totalTime proxy: oidInteger class: theClass name. 
		].
	
	self log magma readCount: readCount totalMs: totalTime reading: (theValue printStringLimitedTo: 70). 

	 
  

 

! !

!WAMagmaRSLogging methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 07:14'!
testRead: oid 
    wantedAt: currentDepth
		   to: minDepth
 forAttribute: varIndex
ofClassWithId: classId
	  version: classVersion
 
| newMinDepth  theClass theAttributeName theValue |					
		
timeStamp := Time millisecondClockValue.

newMinDepth :=  super testRead: oid wantedAt: currentDepth
						to: minDepth
						forAttribute: varIndex
						ofClassWithId: classId
						version: classVersion.

newMinDepth > currentDepth ifFalse: [ self logNoDecisions ifFalse:  [ ^ newMinDepth ] ].
 
entryIndex := entryIndex + 1.
 
theClass := session serializer classIdManager classForId: classId ifAbsent: [ ^self error: 'could not find class id' ].
 
theAttributeName := theClass allInstVarNames at: varIndex  ifAbsent: [ varIndex asString ].
 
theValue := session serializer objectWithOid: oid.  
theValue maIsMutatingProxy 
	ifTrue: [ theValue := ('proxy:',oid printString)]
	ifFalse: [ theValue := theValue class name, ' ',  (theValue  printStringLimitedTo: 40) ].

self log magma << entryIndex << ': [' << (totalTime + timeStamp - readStart) << 'ms] (' << currentDepth << $/ << minDepth << $/ << newMinDepth << ') ' 
				<< theClass name << $- << theAttributeName << ' = ' <<   theValue << ((newMinDepth > currentDepth) ifFalse: [ ' X' ] ifTrue: ['']).

^ newMinDepth ! !

WAMagmaRSLogging subclass: #WAMagmaRSLoggingAll
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ReadStrategy'!

!WAMagmaRSLoggingAll commentStamp: 'kph 12/10/2007 07:46' prior: 0!
This adds logging to the default magma seaside read strategy (using the Logging package). Includes logging of  non-reifications signified by an 'X'. !

!WAMagmaRSLoggingAll methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 07:10'!
logNoDecision

	^ true! !

WAMagmaRSTraditional subclass: #WAMagmaRSDisable
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ReadStrategy'!

!WAMagmaRSDisable commentStamp: 'kph 12/10/2007 08:09' prior: 0!
Disable Read Strategy (set to nil)!

!WAMagmaRSDisable class methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 07:53'!
newFor: helper in: magmaSession

	^ nil! !

WAMagmaRSTraditional subclass: #WAMagmaRSMinimal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ReadStrategy'!

!WAMagmaRSMinimal commentStamp: 'kph 12/10/2007 08:10' prior: 0!
Minimal Read Strategy (magma defaults only)!

!WAMagmaRSMinimal class methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 08:09'!
newFor: helper in: magmaSession

	^ self basicNew session: magmaSession; initialize! !

!WAMagmaRSTraditional class methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 06:01'!
configurationComment

^ self class comment! !

!WAMagmaRSTraditional class methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 09:15'!
newFor: helper in: magmaSession

	^ helper readStrategyInitialize: (self basicNew session: magmaSession; initialize)! !

!WAMagmaRSTraditional methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 06:37'!
forVariablesNamed: varsString onAny: aClass readToDepth: anInteger

	"compatibility"

	(varsString findTokens: ' ') do: [ :var | self forVariableNamed: var onAny: aClass readToDepth: anInteger ] ! !

!WAMagmaRSTraditional methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 06:23'!
initialize

	super initialize.
	self reset! !

!WAMagmaRSTraditional methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 05:58'!
initializeDefaultSpecifications

	super initializeDefaultSpecifications.
	
	"subclasses, specialize and add defaults here"! !

!WAMagmaRSTraditional methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 06:07'!
renderOn: html

html text: ' minimumDepth: '; text: self minimumDepth; break.

depthSpecifications keysAndValuesDo: [ :k :v |  html strong: k; text: ' : '; text: v; break ].! !

!WAMagmaRSTraditional methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 06:23'!
reset
! !

!WAMagmaRSTraditional methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 06:01'!
session: ms

	"for logging, it helps to have the session since the session can convert id's to class names"
	session := ms! !

!MaObjectBuffer methodsFor: '*magma seasideHelper-override' stamp: 'kph 7/18/2007 11:49'!
appendGraphUsing: aMaReadStrategy into: aMaSerializedGraphBuffer currentDepth: currentDepth minDepth: minDepth with: alreadyAppendedSet filer: aMaObjectFiler

	| classId classVersion |

	classId := self classId.
	classVersion := self classVersion.

	1
		to: self objectInstSize
		do:
			[ :varIndex | | oid |

			((MaOidCalculator isOidForOptimizedObject: (oid _ self maInstVarAt: varIndex)) 
			)

				ifFalse:
					[ | newMinDepth | 
			 
				(newMinDepth := aMaReadStrategy 
						testRead: oid
						wantedAt: currentDepth
						to: minDepth
						forAttribute: varIndex
						ofClassWithId: classId
						version: classVersion) > currentDepth  ifTrue: [
						
							self
								appendGraphNode: oid
								into: aMaSerializedGraphBuffer
								using: aMaReadStrategy
								currentDepth: currentDepth
								minDepth: newMinDepth
								with: alreadyAppendedSet
								filer: aMaObjectFiler 
							].
					  ]
		]
 ! !

WAConfigurationAttribute subclass: #WAMagmaLocationAttribute
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Configuration'!

!WAMagmaLocationAttribute methodsFor: 'as yet unclassified' stamp: 'brp 11/4/2005 10:00'!
accept: aVisitor with: anObject

	^ aVisitor visitMagmaLocationAttribute: self with: anObject! !

!WAMagmaLocationAttribute methodsFor: 'as yet unclassified' stamp: 'kph 12/7/2007 22:52'!
defaultLocationFor: app

	^ MagmaLocalLocation defaultFor: app! !

!WAMagmaLocationAttribute methodsFor: 'as yet unclassified' stamp: 'kph 5/24/2007 11:55'!
options

	^ MagmaLocation allSubclasses! !

!MagmaCollection class methodsFor: '*magma seasidehelper' stamp: 'kph 12/9/2007 10:18'!
getVirtualRootIn: root
	
	"Get the view-root of this pov from an actual root"
	
	^  root at: self key! !

!MagmaCollection class methodsFor: '*magma seasidehelper' stamp: 'kph 12/9/2007 10:18'!
key

	^ self name! !

!MagmaCollection class methodsFor: '*magma seasidehelper' stamp: 'kph 12/9/2007 10:19'!
setVirtualRootIn: root to: anObject
	
	"Set the view-root of this pov"
 	 ^  root at: self key put: anObject! !

!MagmaCollection methodsFor: '*magma seasideHelper' stamp: 'kph 12/8/2007 01:37'!
firstRead: helper! !

!MagmaCollection methodsFor: '*magma seasideHelper' stamp: 'kph 12/8/2007 01:37'!
isCached

	^ false! !

!MagmaRepositoryController class methodsFor: '*magma seasideHelper' stamp: 'kph 8/24/2006 22:21'!
openControllers

^OpenControllers! !

!MaBasicReadStrategy methodsFor: '*magma seasideHelper' stamp: 'kph 6/14/2007 00:02'!
signalBeginningToRead: oidInteger classId: classId! !

!MaBasicReadStrategy methodsFor: '*magma seasideHelper' stamp: 'kph 6/22/2007 16:16'!
testRead: oid wantedAt: currentDepth
			to: minDepth
			forAttribute: varIndex
			ofClassWithId: classId
			version: classVersion

| newMinDepth attrDepth |

	attrDepth := self 
						depthOfAttribute: varIndex
						onClassWithId: classId
						version: classVersion.
	newMinDepth := 
		attrDepth > 0 	ifTrue: [ attrDepth + currentDepth max: minDepth ]
						ifFalse: [ currentDepth max: (minDepth + attrDepth "negative") ].
	
	
	^ newMinDepth
! !

!MagmaRemoteLocation methodsFor: '*magma seasidehelper' stamp: 'kph 12/7/2007 22:04'!
renderContentOn: html

	html tableRow: [  
		html tableData id: 'label'; with: 'host'.
		html tableData: [ html textInput on: #host of: self ]
	].
					
	html tableRow: [  
		html tableData id: 'label'; with: 'port'.
		html tableData: [ html textInput value: self port;
							callback: [:v | self port: v asNumber] 
		].
	].
		 ! !

WADispatcherControlPanel subclass: #WAMagmaControlPanel
	instanceVariableNames: 'repositoryControlPanel textInput doneIt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ControlPanel'!

!WAMagmaControlPanel commentStamp: 'kph 10/29/2007 21:49' prior: 0!
Architecture:

This control panel is made available as a plug-in to the seaside/config disptacher editor. The link 'Magma' appears next to applications that have the group #magma. This control panel has to manage its own magmaSession. It does this by creating a psuedo seaside application, local to this control panel which has a magmaSession. It also registers for finalization in order to safely dispose of the psuedo application and to disconnect the magmaSession.

This control panel anticipates that each application using magma may have its own repository control/status panel class. The root object of the repository is sent #magmaControlPanelOn: <magmaSession> to obtain a control/status panel component, or a descriptive string. This is displayed as a sub-part of the WAMagmaControlPanel.

Repository initialization is now expected to be automatic.

[to do]
seasidehosting currently requires the user to manually create the repository directory in the filesystem.



!

!WAMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 7/27/2007 02:12'!
initialize
	"for seaside version<=2.7"
 
	 ^ self register! !

!WAMagmaControlPanel class methodsFor: 'as yet unclassified' stamp: 'kph 7/27/2007 02:06'!
label
	"for seaside version<=2.7"
 
	 ^ 'Magma'! !

!WAMagmaControlPanel methodsFor: 'all stats' stamp: 'kph 9/18/2006 04:18'!
cachedObjectsByClassForSession: aSession

^String streamContents: [ :stream |
	stream nextPutAll: 'cachedObjectCountByClass: '.
	aSession cachedObjectCountByClass 
		do: [ :assoc |
			stream nextPutAll: assoc printString.
			stream nextPutAll: ' '; cr.
		]
]! !

!WAMagmaControlPanel methodsFor: 'all stats' stamp: 'kph 7/26/2007 22:16'!
children

^ Array with: repositoryControlPanel  ! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 7/26/2007 22:17'!
createRepository: repositoryClass
	self appSession unregistered.
	appSession := nil.
	repositoryControlPanel := null.
	
 	MagmaRepositoryController create: self location root: repositoryClass new.
! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 7/26/2007 22:18'!
deleteDatabase

| message1 message2 |

message1 := 'Are you sure that you want to delete repository: ', self location path.
message2 := 'Go ahead and delete repository: ',  self location path.

((self confirm: message1) and: [ self confirm: message2 ]) 
	ifTrue: [ 
		self psuedoAppSession expire.
		appSession := nil.
		repositoryControlPanel := null.
		self location delete 
	].! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 9/4/2006 03:15'!
doIt: aString

doneIt := ( Compiler new evaluate: aString in: nil to: nil ) printString.
! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 9/18/2006 04:24'!
explore: aMagmaSession
 
 [ { aMagmaSession. aMagmaSession root } explore ] fork! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 9/23/2006 07:02'!
explore: sessionId inController: mrc

	self explore:(self findMagmaSessionWithId: sessionId inController: mrc)! !

!WAMagmaControlPanel methodsFor: 'all stats' stamp: 'kph 9/20/2006 15:05'!
findMagmaSessionWithId: id inController: mrc

^MagmaSession allInstances detect: [ :ms | (ms id = id) and: [ ms link localRequestServer processor = mrc ] ] ifNone: [ nil ]! !

!WAMagmaControlPanel methodsFor: 'accessing' stamp: 'kph 7/27/2007 02:02'!
initialize

	super initialize.
	repositoryControlPanel := null.! !

!WAMagmaControlPanel methodsFor: 'accessing' stamp: 'kph 9/18/2006 01:27'!
location

^ self application preferenceAt: #location! !

!WAMagmaControlPanel methodsFor: 'accessing' stamp: 'kph 10/29/2007 05:41'!
magmaSession
	(self appSession respondsTo: #magma) ifTrue: [ ^ self appSession magma magmaSession ].
	
	(self appSession respondsTo: #magmaSession) ifTrue: [ ^ self appSession magmaSession ].
	
	self notify: 'Either a Magma helper is not enabled, nor does the configured session class support magma'.
	
! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 8/2/2007 18:56'!
magmaSession: aBlock ifAbsent: notThereBlock

aBlock
	on: MagmaEnvironmentError, InvalidDirectoryError
	do: notThereBlock.  ! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 12/10/2007 13:36'!
renderApplicationStatusOn: html

	| readStrategy |

	self magmaSession.
 	self renderLocationOn: html.
	self location isLocal ifTrue: [  html text: '   '. 
		html small: [ 
			html anchor 
				callback: [ self deleteDatabase ]; 
				title: 'delete the repository from disk';
				with: '(delete)'. 
		]
	].

	html break.


	html strong: 'Seaside Session: '.
	html text: self appSession printString.

	html break.

	html strong: 'Session Helpers: '.
	html text: self appSession parent printString.

	html break.

	html strong: 'Magma Session: '.
	html text: self magmaSession printString.

	html break.

	html strong: 'Magma Session Id: '.
	html text: self magmaSession id.
	html strong: ' User: '.
	html text: self magmaSession userId.
 
	html break.

	html strong: 'Persisted Objects Count: '.
	html text: self magmaSession repositoryController objectCount.
 
	html strong: ' Cached: '.
	html text: self magmaSession cachedObjectCount.

	html break.

	readStrategy := self magmaSession readStrategy.

	html strong: 'ReadStrategy: '.
	html text: (readStrategy ifNil: [ 'No Read Strategy Set' ]) asString.
	html small: [ 
				html anchor callback: [ [ readStrategy explore ] fork  ]; 
					title: 'open explorer in squeak';
					with: '(explore)'.
			].
		
	html break.

	html strong: 'Root Object: '.
	self magmaSession root in: [ :root |
		html text: 'a ';
			text: root class name;
			text: ' ';
			small: [ 
				html anchor callback: [ self explore: self magmaSession ]; 
					title: 'open explorer in squeak';
					with: '(explore)'.
			];
			break;
			small: 'contents:';
			table: [ 
				root explorerContents do: [ :each | html tableRow: [ html tableData: each asString ]] 
			]
	]! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 12/10/2007 04:04'!
renderContentOn: html

	super renderContentOn: html.
	
	html form: [

		self magmaSession: [ self renderApplicationStatusOn: html.
				html horizontalRule.
				html render: self repositoryControlPanel.
				html horizontalRule. 
			] 
			ifAbsent: [ :ex | self renderMagmaError: ex on: html ].

		html horizontalRule.

		html heading level2; with: 'Overall Magma Status'.
	 
		self renderMagmaPreferencesOn: html.
		self renderStatsForAllRepositoriesOn: html.
	

	].

		self renderUtilsOn: html.
! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 9/4/2006 02:01'!
renderEvaluatorOn: html
	self
		renderLabel: 'Text-Input'
		control: [ :fid :mid |
			html textInput
				value: textInput;
				callback: [ :value | textInput := value ].
			html space.
			html submitButton
				onClick: (html updater
					id: mid;
					triggerForm: fid;
					callback: [ :r | r render: textInput ];
					return: false);
				text: 'Submit' ]
		model: textInput
		on: html! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 9/4/2006 02:13'!
renderLabel: aString control: aBlock model: anObject on: html
	| fid mid |
	fid := mid := nil.
	html div class: 'row'; with: [
		html div class: 'label'; with: aString.
		html div class: 'control'; with: [
			html form
				id: (fid := html nextId);
				with: [ aBlock value: fid value: (mid := html nextId) ] ].
		html div class: 'model'; id: mid; with: anObject.
		html div class: 'clear' ]! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 5/24/2007 13:42'!
renderLocationOn: html

	self location isLocal 
		ifTrue: [
			html strong: 'Repository Path Setting: '.
			html text: self location path.
 		]
		ifFalse: [ 
			html strong: 'Repository remote host: '.
			html text: self location host , ':' , self location port
		].
		
! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 12/9/2007 22:47'!
renderMagmaError: ex on: html

	self renderLocationOn: html.
	html break.
	html strong: 'Error: '.
	html text: ex printString  .
	html break.
	self location isLocal 
		ifTrue: [ html emphasis: 'Local repository not yet initialized (initialization is automatic)' 	] 
		ifFalse: [ html emphasis: 'Remote repository not available' ].! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 5/24/2007 13:42'!
renderMagmaPreferencesOn: html

html strong: 'debug proxies '.
html checkbox title: 'print shows proxy'; on: #debugProxies of: MagmaPreferences.
html strong: 'hand status '.
html checkbox title: 'show wait cursor when submitting magma requests'; on: #handStatus of: MagmaPreferences.
html strong: 'debug '.
html checkbox title: 'onError Halt, and 2 day server link timeout'; on: #debug of: MagmaPreferences.! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 12/10/2007 03:59'!
renderServerDoItOn: html
	self
		renderLabel: 'Server Do It'
		control: [ :fid :mid |
			html textInput
				value: textInput;
				callback: [ :value | self doIt: (textInput := value).  ].
			html space.
			html anchor
				onClick: (html updater
					id: mid;
					triggerForm: fid;
					callback: [ :r | r render:textInput ]);
					with: 'do it' ]
		model: doneIt 
		on: html! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 12/10/2007 13:25'!
renderStatsForAllRepositoriesOn: html

| mSession |

html orderedList: [ 
	MagmaRepositoryController openControllers do: [ :mrc |
		
		html listItem: [
			html strong: mrc objectCount.
			html text: ' objects in repository: '.
			 
			 html strong: mrc pathName.
			html orderedList: [
				mrc repository sessions do: [ :client |
					
					mSession := self findMagmaSessionWithId: client sessionId inController: mrc.

					html listItem: [
						mSession ifNil: [ html text: 'session not found' ]
							ifNotNil: [ 
								html strong: 'session id: '.
								html text: client sessionId printString.
								html strong: ' user: '.
								html text: client userId.
								html strong: ' cached objects: '.
								html text: mSession cachedObjectCount.
								html text:' '..
								html small: [ html anchor callback: [ self showCachedObjectsByClassFor: client sessionId inController: mrc ]; 
								with: '(view)'].
							html text: ' '.
							html small: [ html anchor callback: [ self explore: client sessionId inController: mrc ]; with: '(explore)' ].
						]
					].
				]	 
			].			
				"html tableData: [ html text: c numberOfConnectedClients asString ]."
		].
	].
]
	 
 
 


 ! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 12/10/2007 12:18'!
renderUtilsOn: html

	html horizontalRule.
	html heading level2; with: 'Miscellaneous Utilities: '.
	html break.

	html form: [

		html submitButton callback: [ WAMagma withAllSubclassesDo: [ :each | each cleanUp ].
									  MagmaSession cleanUp.
									  repositoryControlPanel := nil ]; text: 'MagmaSession cleanUp'.

		html break.

		html submitButton callback: [ WAMagmaSolo initialize. ]; title: 'New Sessions (Reloads ReadStrategies)';  text: 'Reset Shared Sessions'.

		html break.

		html submitButton callback: [ self saveImageSnapshot.  ]; text: 'Save Server Image'.

	].
	
	html break.

	self hasScriptaculous 
			ifTrue: [ html horizontalRule.  self renderServerDoItOn: html ]
			ifFalse: [ html small: 'Add SULibrary to config app to enable seaside server "doit"' ].
! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 8/10/2006 16:38'!
rendererClass
        ^ WARenderCanvas ! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 10/29/2007 21:46'!
repositoryControlPanel

| root |

repositoryControlPanel ifNil: [
	root := self magmaSession root.
	(root respondsTo: #magmaControlPanelOn: ) 
		ifTrue: [ repositoryControlPanel := root magmaControlPanelOn: self magmaSession ]
].

^repositoryControlPanel! !

!WAMagmaControlPanel methodsFor: 'callback actions' stamp: 'kph 9/4/2006 01:43'!
saveImageSnapshot

 SmalltalkImage current snapshot: true andQuit: false.! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 12/10/2007 13:13'!
showCachedObjectsByClassFor:  sessionId inController: mrc 

 	| form |
	form := WAFormDialog new.
	
	(self findMagmaSessionWithId: sessionId inController: mrc) cachedObjectCountByClass asSortedCollection explorerContents  do: [:each | form addMessage: each asString ].
		
	self call: form.! !

!WAMagmaControlPanel methodsFor: 'rendering' stamp: 'kph 8/11/2006 14:16'!
style

^ 

'
div.repositories table td {
	border: 1px solid #888;

'! !

Object subclass: #WAReadStrategyClassSpec
	instanceVariableNames: 'theClass myDefinition superSpec specificVars myVarsDepth allVarsDepth instVarMap inheritedExtraVarsDepth'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ReadStrategy'!

!WAReadStrategyClassSpec commentStamp: 'kph 6/19/2007 01:35' prior: 0!
The antidote to the brain bending default implementation!

!WAReadStrategyClassSpec class methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 04:07'!
forClass: aClass

	^ (self basicNew) setClass: aClass; initialize; yourself! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 06:15'!
allVarsDepth

	^ allVarsDepth! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 06:38'!
extraVarsDepth

	^ (myVarsDepth ifNil: [ allVarsDepth ]) ifNil: [ inheritedExtraVarsDepth ]! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 14:31'!
inheritDepthsIn: depths using: aMaClassIdManager

	| inheritedDepths superClassDepths |

 
	" Q2Log debug:  self printString. "

	"if we have an answer we dont need to calc one" 
	instVarMap ifNotNil: [ ^ self ].

	"if we have no superclass then use our result"
	
	superClassDepths := depths at: { myDefinition superclassDefinition id.  myDefinition superclassDefinition version } ifAbsent: [ self  ].

	"we are going to inherit from our superclass we make sure that he has inherited"

	superClassDepths ~= self ifTrue:[ superClassDepths inheritDepthsIn: depths using: aMaClassIdManager].
	
	inheritedDepths := superClassDepths instVarMap.
	inheritedExtraVarsDepth := superClassDepths extraVarsDepth.
	 
	instVarMap := self instVarMap.

	instVarMap withIndexDo: [ :d :n | 
		(instVarMap at: n) ifNil: [ 
			instVarMap at: n put: (((inheritedDepths at: n ifAbsent:[nil]) ifNil: [ self extraVarsDepth ]) )
		]
	].


	(instVarMap  includes: nil) ifTrue: [ self error: 'incomplete read strategy' ].
	! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 06:37'!
inheritedExtraVarsDepth: evd

	 inheritedExtraVarsDepth := evd! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 07:10'!
instVarMap

	instVarMap ifNotNil: [ ^ instVarMap ].

	instVarMap := Array new: myDefinition namedInstSize.

	allVarsDepth ifNotNil: [ 
		instVarMap isEmpty ifTrue: [ ^instVarMap := Array with: allVarsDepth ].

		instVarMap atAllPut: allVarsDepth 
	].

	myVarsDepth ifNotNil: [ 
		myDefinition superclassDefinition namedInstSize + 1 to: instVarMap size do: [ :n |
			instVarMap at: n put: myVarsDepth 
		].
 	].

	specificVars ifNotNil: [ 
		specificVars keysAndValuesDo: [ :var :depth |
			instVarMap at: (myDefinition allInstVarNames indexOf: var) put: depth
		]
	].

	^ instVarMap! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 04:42'!
isArray

	^ false! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 02:33'!
makeInstVarMapFor: aClassDefinition

	self setClassDefinition: aClassDefinition.
	"we dont make the map yet, since we have it in mind!!"

	^ self ! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 7/12/2007 16:57'!
printOn: str
	
	myDefinition ifNotNil: [ str	print: { myDefinition className. myDefinition version.} ; nextPutAll: '->';   space ].
	str print: theClass.
	allVarsDepth ifNotNil: [ str nextPutAll: ' all:'; print: allVarsDepth  ].
	myVarsDepth ifNotNil: [ str nextPutAll: ' my:'; print: myVarsDepth; space  ].
	specificVars ifNotNil: [ str print: specificVars ].
	instVarMap ifNotNil: [ str space; print: instVarMap ].

	self extraVarsDepth ifNotNil: [ str nextPutAll: 'scExtras:'; print: self extraVarsDepth; cr ].! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 01:37'!
readAllVarsToDepth: d

	allVarsDepth := d! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 01:37'!
readMyVarsToDepth: d

	myVarsDepth := d! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/22/2007 16:52'!
readVar: var toDepth: d

	(theClass allInstVarNames includes: var) ifFalse: 
		[ MagmaUserError signal: var , ' is not an attribute of ' , var name ].

	(specificVars ifNil: [ specificVars := Dictionary new ]) at: var put: d

	! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 05:37'!
readVars: vars toDepth: d

	(vars findTokens: ' ') do: [ :var | self readVar: var toDepth: d ]	 ! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 02:43'!
setClass: aClass

	"This is the class the definition is originally specified on"
	theClass := aClass! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 02:51'!
setClassDefinition: aClassDefinition

	"this is the definition that this spec is being applied to, i.e. the 
	setClass and any subclasses which do not have their own spec"

	myDefinition := aClassDefinition! !

!WAReadStrategyClassSpec methodsFor: 'as yet unclassified' stamp: 'kph 6/19/2007 02:50'!
setDefinition: aClassDefinition

	"this is the definition that this spec is being applied to, i.e. the 
	setClass and any subclasses which do not have their own spec"

	myDefinition := aClassDefinition! !

Object subclass: #WAServerManagedPreferences
	instanceVariableNames: 'preferences expiry'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Model'!

!WAServerManagedPreferences commentStamp: 'kph 12/9/2007 08:46' prior: 0!
WAServerManagedPreferences is intended to allow a number of seaside servers to be given some of their application's preferences from a central persistent store.

WAServerManagedPreferences is implemented as a module, being as self contained as possible.  

This module allows a seaside server application to pick up settings and preferences from the magma server. Whenever a new seaside session is started, and the actual root is read for the first time only, the helpert's #onFirstRead: method is called. The default behaviour calls 'self rootAs: WAServerManagedPreferences'.

If the expire time has been reached then any persisted preferences are read and the current application's preferences are updated with the values that it contains. The values are each sent #value before being written.

This would allow #location to return a specialized version of MagmaRemoteLocation which when sent #value, evaluates picking a remote server based upon the client's itentity

!

!WAServerManagedPreferences class methodsFor: 'magma db view trait' stamp: 'kph 12/10/2007 12:13'!
getVirtualRootIn: root
	
	"Get the view-root of this pov"
	
	^  root at: self key ifAbsent: nil! !

!WAServerManagedPreferences class methodsFor: 'magma db view trait' stamp: 'kph 12/10/2007 12:13'!
key

	^ self name! !

!WAServerManagedPreferences class methodsFor: 'magma db view trait' stamp: 'kph 12/10/2007 12:13'!
setVirtualRootIn: root to: aServerManagementPreferences
	
	"Set the view-root of this pov"
	
	^ root at: self key put: aServerManagementPreferences! !

!WAServerManagedPreferences methodsFor: 'as yet unclassified' stamp: 'kph 10/29/2007 11:17'!
expiry

	^ expiry! !

!WAServerManagedPreferences methodsFor: 'as yet unclassified' stamp: 'kph 12/8/2007 01:50'!
expiry: dateAndTime

	expiry := dateAndTime! !

!WAServerManagedPreferences methodsFor: 'as yet unclassified' stamp: 'kph 12/8/2007 01:50'!
firstRead: helper

	expiry ifNil: [ ^ self ].
	
	expiry < DateAndTime now ifTrue: [ self update: helper ]! !

!WAServerManagedPreferences methodsFor: 'as yet unclassified' stamp: 'kph 12/8/2007 01:49'!
initialize
	expiry := nil.
	preferences := IdentityDictionary new.! !

!WAServerManagedPreferences methodsFor: 'as yet unclassified' stamp: 'kph 10/29/2007 12:27'!
isCached

	^ false! !

!WAServerManagedPreferences methodsFor: 'as yet unclassified' stamp: 'kph 10/29/2007 11:18'!
preferences

	^ preferences! !

!WAServerManagedPreferences methodsFor: 'as yet unclassified' stamp: 'kph 10/29/2007 12:43'!
update: helper

	helper seasideSession application in: [ :app |
		self preferences keysAndValuesDo: [ :k :v | app preferenceAt: k put: v value ]
	]! !

WASystemConfiguration subclass: #WAMagmaConfiguration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Configuration'!

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 6/12/2007 14:34'!
ancestors

	^ #() "Array with: (WARenderLoopConfiguration new)"! !

!WAMagmaConfiguration methodsFor: 'helper class' stamp: 'kph 12/9/2007 13:51'!
attributeHelperClass

	^ (WAListAttribute key: (#magmaHelperClass, self suffix) asSymbol group: self group)
		options: WAMagma withAllSubclasses asArray; 
		yourself ! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:43'!
attributeLocation

	^ WAMagmaLocationAttribute key: (#location, self suffix) asSymbol group: self group! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:43'!
attributeReadStrategy

	^ (WAListAttribute key: (#readStrategy, self suffix) asSymbol group: self group)
		options: self readStrategyClasses; 
		yourself
		! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:43'!
attributeSignalProxyMaterializations

	^ WABooleanAttribute key: (#signalProxyMaterializations, self suffix) asSymbol group: self group
! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 10/27/2007 04:56'!
attributes
	^ (self class selectors
		select: [ :each | each ~= #attributes
			and: [ each beginsWith: #attribute ] ])
		collect: [ :each | self perform: each ]! !

!WAMagmaConfiguration methodsFor: 'helper class' stamp: 'kph 12/9/2007 04:37'!
commentMagmaHelperClass: value config: configuration
 
	^ value configurationComment! !

!WAMagmaConfiguration methodsFor: 'helper class' stamp: 'kph 12/10/2007 07:48'!
commentReadStrategy: value config: configuration
 
	^ value configurationComment! !

!WAMagmaConfiguration methodsFor: 'helper class' stamp: 'kph 12/9/2007 05:44'!
group

	^ #magma! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 12/7/2007 22:51'!
location

	^  nil! !

!WAMagmaConfiguration methodsFor: 'helper class' stamp: 'kph 12/9/2007 06:52'!
magmaHelperClass
		
	^ WAMagmaSoloAuto! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 05:48'!
readStrategy 

^ WAMagmaRSDefault 
! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 08:08'!
readStrategyClasses 

	^ WAMagmaRSTraditional withAllSubclasses 
! !

!WAMagmaConfiguration methodsFor: 'as yet unclassified' stamp: 'kph 5/24/2007 14:12'!
signalProxyMaterializations 

	^ false! !

!WAMagmaConfiguration methodsFor: 'helper class' stamp: 'kph 12/9/2007 05:42'!
suffix

	^ ''! !

WAMagmaConfiguration subclass: #WAMagmaConfiguration2nd
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Configuration'!

!WAMagmaConfiguration2nd methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:46'!
attributes
	^ (self class superclass selectors
		select: [ :each | each ~= #attributes
			and: [ each beginsWith: #attribute ] ])
		collect: [ :each | self perform: each ]! !

!WAMagmaConfiguration2nd methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:47'!
commentMagmaHelperClass2: value config: configuration
 
	^ 'Second magma instance is available via "<code>self session magma2</code>" <br> <br>', value configurationComment! !

!WAMagmaConfiguration2nd methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:47'!
group

	 ^  #'magmaNo.2'  ! !

!WAMagmaConfiguration2nd methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:23'!
magmaHelperClass2
		
	^ WAMagmaMock! !

!WAMagmaConfiguration2nd methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 05:48'!
readStrategy2

^ WAMagmaRSDefault 
! !

!WAMagmaConfiguration2nd methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:38'!
signalProxyMaterializations2

	^ false! !

!WAMagmaConfiguration2nd methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:41'!
suffix

	 ^  '2'! !

!MagmaLocalLocation class methodsFor: '*magma seasidehelper' stamp: 'kph 12/7/2007 22:29'!
defaultFor: seasideApp

	^ self path: ('magma-', seasideApp name)! !

!MagmaLocalLocation methodsFor: '*magma seasidehelper' stamp: 'cmm 7/1/2007 13:52'!
commitLogDirectory
	^ FileDirectory on: (self directory fullNameFor: MaRecoveryManager commitLogDirectoryName)! !

!MagmaLocalLocation methodsFor: '*magma seasidehelper' stamp: 'cmm 7/16/2007 22:12'!
delete
	"Delete the files at my location."
	self isRepository 
		ifTrue: 
			[ MagmaDeleteWarning signal: 'The Magma database at ' , self pathName , ' is about to be deleted.'.
			self directory deleteLocalFiles.
			self commitLogDirectory deleteLocalFiles ]
		ifFalse: 
			[ MagmaNotification signal: 'There is no Magma repository at ' , self pathName ]! !

!MagmaLocalLocation methodsFor: '*magma seasidehelper' stamp: 'cmm 4/20/2007 16:31'!
isRepository
	^ self directory exists and: [ self directory fileExists: MaObjectFiler objectsPhysicalFilename ]! !

!MagmaLocalLocation methodsFor: '*magma seasidehelper' stamp: 'kph 12/7/2007 22:46'!
isValid

	^ path notNil! !

!MagmaLocalLocation methodsFor: '*magma seasidehelper' stamp: 'kph 12/7/2007 21:42'!
renderContentOn: html
 
	html tableRow: [  
			html tableData id: 'label'; with: 'path:'.
			html tableData: [ html textInput on: #path of: self ]
	]
			 ! !

!UnixFileDirectory methodsFor: '*magma seasideHelper' stamp: 'kph 12/10/2007 07:30'!
directoryExists: filenameOrPath
    "Handles the special case of testing for the root dir: there isn't a
    possibility to express the root dir as full pathname like '/foo'."

	"I am really not sure why this fix is here. I think it is for Seaside Hosting.
	
	KPH fix for seaside hosting. This enables assureExistence to work
	by assuming that our default directory exists we do not go outside
	of the allowed filesystem"

    ^ filenameOrPath = '/'
        or: [ filenameOrPath = self class default pathName
        or: [ super directoryExists: filenameOrPath ] ]
! !

!WAApplicationEditor methodsFor: '*magma seasidehelper' stamp: 'kph 12/7/2007 22:52'!
visitMagmaLocationAttribute: anAttribute with: html

	| location |
	location := configuration valueForAttribute: anAttribute. 

	location ifNil: [ location := anAttribute defaultLocationFor: application ].
		
	html select 
		list: anAttribute options;
		selected: location class;
		beSubmitOnChange;
		labels: [ :ea | ea name ];
		callback: [ :item | 	(configuration valueAt: #location) class == item
			ifFalse: [ configuration takeValue: (item defaultFor: application) forAttribute: anAttribute. ]
 		];
		yourself.

	location renderOn: html  ! !

IdentityDictionary subclass: #WADictionaryRoot
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Model'!

!WADictionaryRoot commentStamp: 'kph 12/7/2007 23:12' prior: 0!
Note that WADictionaryRoot is a subclass of IdentityDictionary. Magma does not support subclasses of IdentityDictionary which add instance vars.!

!WADictionaryRoot class methodsFor: 'magma db view trait' stamp: 'kph 12/10/2007 09:05'!
buildReadStrategyOn: aReadStrategy

	! !

!WADictionaryRoot class methodsFor: 'magma db view trait' stamp: 'kph 12/9/2007 09:13'!
getVirtualRootIn: root
	
	"Get the view-root of this pov from an actual root"
	
	^  root at: self key! !

!WADictionaryRoot class methodsFor: 'magma db view trait' stamp: 'kph 12/9/2007 09:13'!
key

	^ self name! !

!WADictionaryRoot class methodsFor: 'magma db view trait' stamp: 'kph 12/9/2007 09:13'!
setVirtualRootIn: root to: anObject
	
	"Set the view-root of this pov"
 
	 ^ root at: self key put: anObject ! !

!WADictionaryRoot methodsFor: 'as yet unclassified' stamp: 'kph 12/8/2007 01:36'!
firstRead: helper

 ! !

!WADictionaryRoot methodsFor: 'as yet unclassified' stamp: 'kph 12/8/2007 01:35'!
isCached

	^ true! !

WASessionHelper subclass: #WAMagma
	instanceVariableNames: 'seasideSession magmaSession root views suffix monitor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Helpers'!

!WAMagma commentStamp: 'kph 12/9/2007 08:36' prior: 0!
The aim of this version of the Magma-seaside integration framework is to learn some lessons from projects already using Magma in order to satisfy the following goals.

The overriding goal was to make the tutorial very easy, so as to blow away the competition!!

====
1. Simplify configuration and customization.

2. Ease combining Magma with other seaside technologies, to which end we do not subclass WASession.

3a. Enable a choice of session management options such as individual, shared or pooled sessions.
3b. Support common Magma-Seaside arrangements out of the box (e.g. Solo mode where Seaside is the only user).
3c. Support a second magma configuration option, to support combinations of session management types.
	e.g. a shared session for read only access or login validation.
 
4a. Facilitiate modularization, such that a Magma "user-accounts" module can be published as a standalone package.
	Modules are combinable with other modules, such as a "Pier module", or a "shopping-cart" module.
4b. Similarly, facilitate combination of projects having separate databases into one, e.g. Gjallar & Pier.

5. Simplify use and initialization, not just of the root object but also sub-view-roots in the database
   which are used by different application-modules or application sub-domains.
 
6. Facilitate server managed preferences and data, multi-server operation, and load sharing schemes.

====
Wiki documentation page at:  http://wiki.squeak.org/squeak/6019

If a model is retrieved that is not of the correct class, #rootAs: attempts to mutate the instance into the desired class, thus providing some automatic schema migration at the module level.

6. Server managed preferences

This is provided by an optional module. It allows a seaside server application to pick up settings and preferences from the magma server. Whenever a new seaside session is started, on the first time only, the root object's #firstRead: method is called. If it then calls 'helper rootAs: WAServerManagedPreferences'.

WAServerManagedPreferences is implemented as a module, being as self contained as possible. It assumes the root object of the database is a dictionary (if not it will need to be reconfigured appropriately).

This module, when #firstRead:, checks its #expiry time, and if it has expired, it loads the current application's preferences with the values that it contains. The values are each sent #value before being written.

This would allow #location to return a version of MagmaRemoteLocation which when sent #value, evaluates picking a remove server based upon the client's itentity


Direct usage:
WAMagma helper may be used with Seaside, or directly. Configuration is per-seaside application e.g.

(WAMagma forApp: 'todo') location.
(WAMagma forApp: 'todo') readStrategyClass.
(WAMagma forApp: 'todo') areMaterializationNotificationsEnabled.
(WAMagma forApp: 'todo') commit: [].
 


!

!WAMagma class methodsFor: 'helper protocol' stamp: 'kph 12/9/2007 14:15'!
cleanUp

	^ self initialize! !

!WAMagma class methodsFor: 'public factory methods' stamp: 'kph 12/9/2007 11:26'!
configurationComment
	
^ 'Basic Magma Support. Each Seaside Session gets its own Magma session. Currently it takes a few seconds to obtain a fresh Magma session so it is a good idea if your app can avoid using it on the first hit, if not consider using a shared helper for the first hit. After the first page has been rendered and sent to the browser, we begin to obtain our Magma session in anticipation of the next hit.'! !

!WAMagma class methodsFor: 'magma preferences' stamp: 'kph 10/28/2007 23:50'!
debugProxies: aBoolean

	MagmaPreferences debugProxies: aBoolean! !

!WAMagma class methodsFor: 'public factory methods' stamp: 'kph 12/10/2007 12:14'!
defaultDatabaseRootClass 

	^ WADictionaryRoot! !

!WAMagma class methodsFor: 'public factory methods' stamp: 'kph 12/8/2007 00:15'!
forApp: appName 

	"this gets you DIRECT command line access to the helper outside of browser
	with the configuration of the named application"
	
	^ (WASession forApp: appName) magma 
	 ! !

!WAMagma class methodsFor: 'helper protocol' stamp: 'kph 12/8/2007 00:05'!
helperKey

	^ #magma! !

!WAMagma class methodsFor: 'helper protocol' stamp: 'kph 10/27/2007 05:40'!
isEnabledFor: app

	^ (app preferenceAt: #magmaHelperClass)  = self ! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 10/29/2007 00:40'!
abort

	self magmaSession abort! !

!WAMagma methodsFor: 'private' stamp: 'kph 12/9/2007 13:48'!
addHelperTo: helpers

	| second |
	
	super addHelperTo: helpers.

	second := helpers at: (self class helperKey, '2') asSymbol put: ((helpers application preferenceAt: #magmaHelperClass2) ifNil: [ ^ self ]) new.
	second 
		beSecondMagmaHelper;
	 	setUp: helpers.! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 10/28/2007 23:31'!
backupDatabaseTo: aDirectory
	" Backup the database to aDirectory, and wait until the backup process ends. "

	| semaphoreWaitingForEndOfBackupProcess |
	semaphoreWaitingForEndOfBackupProcess := (self magmaSession repositoryController fullBackupTo: aDirectory).
	semaphoreWaitingForEndOfBackupProcess wait! !

!WAMagma methodsFor: 'private' stamp: 'kph 12/9/2007 06:04'!
beSecondMagmaHelper

	suffix := '2'! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 10/29/2007 00:39'!
begin

	self magmaSession begin! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 10/28/2007 23:29'!
canStubOut: anObject

	^ self magmaSession canStubOut: anObject
! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 12/9/2007 11:38'!
commit

	self magmaSession commit! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 12/8/2007 00:46'!
commit: aBlock

	"we wrap the call to commit so that 'direct' users can make use of the factory"
	
	^ self privateDuring: [ self magmaSession commit: aBlock ]! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 12/9/2007 11:40'!
commitAndBegin

	self inTransaction ifFalse: [ self begin ].
	self magmaSession commitAndBegin! !

!WAMagma methodsFor: 'configuration' stamp: 'kph 12/9/2007 13:50'!
configurationHelperClass

 	^ seasideSession application preferenceAt: (#magmaHelperClass , suffix) asSymbol! !

!WAMagma methodsFor: 'configuration' stamp: 'kph 12/9/2007 13:49'!
configurationLocation

	^ seasideSession application preferenceAt: (#location , suffix) asSymbol! !

!WAMagma methodsFor: 'configuration' stamp: 'kph 12/9/2007 13:50'!
configurationReadStrategyClass

 ^ seasideSession application preferenceAt: (#readStrategy , suffix) asSymbol! !

!WAMagma methodsFor: 'configuration' stamp: 'kph 12/9/2007 13:50'!
configurationSignalProxyMaterializations

 ^ seasideSession application preferenceAt: (#signalProxyMaterializations,  suffix) asSymbol! !

!WAMagma methodsFor: 'seaside session' stamp: 'kph 12/10/2007 12:54'!
id

	^ 'seaside-', self seasideSession application name,'-', Time now print24.! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 12/9/2007 22:22'!
initialize
	views := IdentityDictionary new.
	monitor := Monitor new.
	suffix := ''.
! !

!WAMagma methodsFor: 'configuration' stamp: 'kph 12/9/2007 06:09'!
location

	"perhaps cache this value here"
	^ self configurationLocation! !

!WAMagma methodsFor: 'magma sessions' stamp: 'kph 12/9/2007 22:32'!
magmaSession

	magmaSession ifNotNil: [ ^ magmaSession ].

	"we need to obtain a magmaSession, only one can be obtained at a time for this helper.
	 when we get our chance, magmaSession may have already been obtained"
	
	^ self monitor critical: [ magmaSession ifNil: [ magmaSession := self obtainMagmaSession ] ]! !

!WAMagma methodsFor: 'private' stamp: 'kph 12/9/2007 22:32'!
monitor

	^ monitor! !

!WAMagma methodsFor: 'magma sessions' stamp: 'kph 12/10/2007 12:52'!
obtainMagmaSession
 
	| ms |
	
	ms := self location newSessionAs: self id.
	ms preferences signalProxyMaterializations: (self configurationSignalProxyMaterializations).
	self readStrategyUpdateFor: ms.


	^ ms
 ! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 12/8/2007 00:53'!
onFirstRead

	self rootAs: WAServerManagedPreferences. ! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 12/8/2007 00:20'!
possiblyCommit: aBlock
	"This is a variant of #commit: in which we do #begin
	and then #commit if still #inTransaction. This makes
	it possible for code inside the block to abort and perform
	its own non-nested commits instead. We only commit at
	the end if we are still in a transaction.
 "

	^ self privateDuring: [ 
		| result |
		self magmaSession begin.
		result := aBlock value.
		self magmaSession inTransaction ifTrue: [self magmaSession commit].
		result]! !

!WAMagma methodsFor: 'seaside hooks' stamp: 'kph 12/9/2007 11:19'!
postProcessRequest: aRequest returned: aResponse

 	"occurs after response has been sent to the server"

	"if the first hit has not obtained a MagmaSession, let us begin to get one. 
	 the semaphores in #magmaSession should prevent the second hit re-entering"
	
	 self magmaSession
	! !

!WAMagma methodsFor: 'private' stamp: 'kph 12/8/2007 00:19'!
privateDuring: aBlock

	"we wrap the block as an aid so that 'direct' users can make use of the factory"
	
	^ WACurrentSession use: seasideSession during: [ aBlock value ]! !

!WAMagma methodsFor: 'private' stamp: 'kph 12/8/2007 01:02'!
privateInitializeRootTo: aValue

	self location isLocal ifFalse: [ ^ self error: ' cannot create a non local database' ].
	 
	self privateDuring: [ 
		MagmaRepositoryController
			create: self location
			root: aValue
	].

	^ aValue! !

!WAMagma methodsFor: 'magma sessions' stamp: 'kph 12/10/2007 12:08'!
readStrategyAddFor: viewClass
	
	"when reading a new (i.e. not yet cached viewClass give it a peck at the readStrategy)."
	
	(viewClass respondsTo: #buildReadStrategyOn:) ifFalse: [ ^ self ].
	
	viewClass buildReadStrategyOn: magmaSession readStrategy! !

!WAMagma methodsFor: 'magma sessions' stamp: 'kph 12/10/2007 12:06'!
readStrategyInitialize: aReadStrategy

	"set our initial read strategy"
	self class defaultDatabaseRootClass buildReadStrategyOn: aReadStrategy.
	
	^ aReadStrategy! !

!WAMagma methodsFor: 'magma sessions' stamp: 'kph 12/10/2007 11:55'!
readStrategyUpdateFor: ms

	 ^ ms readStrategy: (self configurationReadStrategyClass newFor: self in: ms) ! !

!WAMagma methodsFor: 'seaside hooks' stamp: 'kph 12/9/2007 05:29'!
responseForRequest: aRequest wrap: aBlock

	| result |
	 
	^ [ "before"   result := aBlock value. "after". result ]
! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 12/10/2007 09:13'!
rootAs: interfaceClass

	| viewRoot |
	"
	It presents a root one of many possible views from the point of view of this application, as defined by the given interfaceClass 
	
	Views are sub-sections of the database, such that one database can be the combination of databases from 
	several separate applications combined.  
	
	When we obtain a root, or view-root, If it is nil we initialize it to be an instance of the nominated class.
	
	If we find that the database has no real root object we initialize it to #defaultRootClass.
	 
	If we find that the root class is not the class we anticipated we coerce it to given class  
	(of course this will only work if the two are similar)
	"
	
	"hit our cache first"
	views at: interfaceClass ifPresent: [ :v | ^ v ].
	
	"obtain the actual database root, if the database does not exist create it using the nominated root class" 
			
	root ifNil: [
		
		root := [ self magmaSession root ] on: MagmaEnvironmentError do: [ :ex | self privateInitializeRootTo: self class defaultDatabaseRootClass new ].
		"we are at the first read of the actual db root"
		self onFirstRead.
	].

	root ifNil: [ self error: 'database is empty, or has yet to be configured' ].

	"if we were handed nil as the interfaceClass return the actual database root"
	interfaceClass ifNil: [ ^ root ].

	"before we read the requested root, update the readStrategy"
	self readStrategyAddFor: interfaceClass.
	
	"map the actual root to the root of this view"
	viewRoot := interfaceClass getVirtualRootIn: root.

	viewRoot == root ifTrue: [ ^ viewRoot ].
	
	"if it doesnt exist initialize it"
	viewRoot ifNil: [ self commit: [ viewRoot := interfaceClass setVirtualRootIn: root to: interfaceClass new ] ] .
	
	"if it is of the wrong class attempt to migrate it"
	(viewRoot class = interfaceClass) ifFalse: [ 
			self commit: [ viewRoot := interfaceClass setVirtualRootIn: root to:  (interfaceClass adoptInstance: viewRoot from: viewRoot class) ] ]  .
	
	viewRoot firstRead: self.
	
	viewRoot isCached ifTrue: [ views at: interfaceClass put: viewRoot ].

	^ viewRoot
	! !

!WAMagma methodsFor: 'seaside session' stamp: 'kph 10/29/2007 12:04'!
seasideSession

	^ seasideSession! !

!WAMagma methodsFor: 'lifecycle' stamp: 'kph 12/9/2007 05:20'!
setUp: helpers
	 seasideSession := helpers session.! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 10/28/2007 23:37'!
stubOut: anObject

	^ self magmaSession stubOut: anObject
! !

!WAMagma methodsFor: 'magma operations' stamp: 'kph 10/29/2007 00:42'!
submit: serverRequest

	^ self magmaSession submit: serverRequest 
! !

!WAMagma methodsFor: 'lifecycle' stamp: 'kph 12/9/2007 05:18'!
tearDown! !

WAMagma subclass: #WAMagmaBasicPool
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Helpers'!

WAMagmaBasicPool class
	instanceVariableNames: 'pools poolFillSemaphores'!

!WAMagmaBasicPool commentStamp: 'kph 12/9/2007 13:29' prior: 0!
Similar to the basic Magma Support provided by WAMagma. Each Seaside Session gets its own Magma session. However a pool of available MagmaSessions is generated in the background so as to be available as soon as a new Seaside session requires one. The replenishment of the pool is triggered after each web hit. MagmaSessions from expired SeasideSessions are recycled back into the pool.!

WAMagmaBasicPool class
	instanceVariableNames: 'pools poolFillSemaphores'!

!WAMagmaBasicPool class methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:46'!
cleanUp

	pools := nil.
	poolFillSemaphores := nil! !

!WAMagmaBasicPool class methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 12:24'!
configurationComment 

	^  self class comment! !

!WAMagmaBasicPool class methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 12:18'!
defaultPoolSize

	^ 5! !

!WAMagmaBasicPool class methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 12:33'!
maxPoolSize

	^ 10! !

!WAMagmaBasicPool class methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:45'!
poolFillSemaphores

	^ poolFillSemaphores ifNil: [ poolFillSemaphores := Dictionary new ]! !

!WAMagmaBasicPool class methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:46'!
pools

^ pools ifNil: [ pools := Dictionary new.  ]! !

!WAMagmaBasicPool methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:55'!
fillPool: pool

	| pfs |

	pool size < self class defaultPoolSize ifFalse: [ ^ self ].
	
	pfs := self class poolFillSemaphores at: self location ifAbsentPut: [ Semaphore new ].
	
	pfs critical: [
		
		[ pool size to: self class defaultPoolSize do: [ pool add: super obtainMagmaSession ] ] forkAt: ProcessorScheduler userBackgroundPriority.
	
	] ifLocked: [ "someone else is already filling the pool- do nothing" ]! !

!WAMagmaBasicPool methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:28'!
initialize

	super initialize.
	poolMonitor := Monitor new.! !

!WAMagmaBasicPool methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:38'!
obtainMagmaSession

	| pool |
	
	pool := self pool.
	
	^ self pool isEmpty
		ifTrue: [ super obtainMagmaSession ]
		ifFalse: [ pool removeFirst abort "refresh it since it has been sitting in the pool" ].
			! !

!WAMagmaBasicPool methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 12:36'!
pool 

	^ self class pools at: self location ifAbsentPut: [ OrderedCollection new ]! !

!WAMagmaBasicPool methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:55'!
postProcessRequest: aRequest returned: aResponse

 	"occurs after response has been sent to the server"
	
	self fillPool: self pool! !

!WAMagmaBasicPool methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:57'!
tearDown

	| pool |

	((magmaSession isNil or: [ magmaSession isOpen not ]) or: [ magmaSession isConnected not]) ifTrue: [ ^ self ].
	
	pool := self pool.

	pool size < self class maxPoolSize ifTrue: [ pool add: magmaSession ]! !

WAMagma subclass: #WAMagmaMock
	instanceVariableNames: 'readStrategy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Helpers'!

!WAMagmaMock class methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 02:50'!
configurationComment
	
	^ 'Mock Magma is useful for testing, it runs without a database present at all. MagmaCollections should still work as before'! !

!WAMagmaMock class methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 02:52'!
configurationCommentAutoCommit: enabled

	^ 'Auto commit is not relevant when running with a mock database.'! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:22'!
abort! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:28'!
backupDatabaseTo: aDirectory

	(Delay forSeconds: 30) wait! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:22'!
begin! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:29'!
canStubOut: a

	^ true! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:22'!
commit

! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:18'!
commit: aBlock
	"Evaluate the block - no transaction."

	^aBlock value! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:30'!
commitAndBegin! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:55'!
initialize
	super initialize.
  
	self readStrategyUpdateFor: self.! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:34'!
location

	^ self! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:51'!
obtainMagmaSession

	"This mock spoofs the features of the other helpers, if the application tries to use the MagmaSession directly we try to offer a TestResourceMockMagma"

	^  (Smalltalk at: #TestResourceMockMagma ifAbsent: [ self error: 'Load Magma sunit - to use TestResourceMockMagma' ]) new
			readStrategy: readStrategy;
			yourself
! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:56'!
readStrategyAddFor: viewClass
	
	"when reading a new (i.e. not yet cached viewClass give it a peck at the readStrategy)."
	
	viewClass buildReadStrategyOn: readStrategy! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:50'!
rootAs: interfaceClass

	| viewRoot |
	"
	It presents a root one of many possible views from the point of view of this application, as defined by the given interfaceClass 
	
	Views are sub-sections of the database, such that one database can be the combination of databases from 
	several separate applications combined.  
	
	When we obtain a root, or view-root, If it is nil we initialize it to be an instance of the nominated class.
	
	If we find that the database has no real root object we initialize it to #defaultRootClass.
	 
	If we find that the root class is not the class we anticipated we coerce it to given class  
	(of course this will only work if the two are similar)
	"
	
	"hit our cache first"
	views at: interfaceClass ifPresent: [ :v |  

		"in the mock all views are cached, so we simulate the behaviour of a non-cached view"		
		v isCached ifFalse: [ v firstRead: self ]. 
		
		^ v  ].
	
	"obtain the actual database root, if the database does not exist create it using the nominated root class" 
			
	root ifNil: [
		
		root := self class defaultDatabaseRootClass new 
		"we are at the first read of the actual db root"
		self onFirstRead.
	].

	"if we were handed nil as the interfaceClass return the actual database root"
	interfaceClass ifNil: [ ^ root ].

	"before we read the requested root, update the readStrategy"
	self readStrategyAddFor: interfaceClass.
	
	"map the actual root to the root of this view"
	viewRoot := interfaceClass getVirtualRootIn: root.

	"if it doesnt exist initialize it"
	viewRoot ifNil: [ self commit: [ viewRoot := interfaceClass setVirtualRootIn: root to: interfaceClass new ] ] .
	
	"if it is of the wrong class attempt to migrate it"
	(viewRoot class = interfaceClass) ifFalse: [ 
			self commit: [ viewRoot := interfaceClass setVirtualRootIn: root to:  (interfaceClass adoptInstance: viewRoot from: viewRoot class) ] ]  .
	
	viewRoot firstRead: self.
	
	"in the mock all views are cached"
	views at: interfaceClass put: viewRoot.

	^ viewRoot
	! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:29'!
stubOut: a
	"Do nothing."! !

!WAMagmaMock methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 11:59'!
submit: serverRequest

	^	serverRequest response! !

WAMagma subclass: #WAMagmaSolo
	instanceVariableNames: 'mutex'
	classVariableNames: 'SharedMagmaSessions SharedMonitors'
	poolDictionaries: ''
	category: 'Magma seasideHelper-Helpers'!

!WAMagmaSolo commentStamp: 'kph 12/9/2007 13:27' prior: 0!
<strong>WAMagmaSolo is specifically designed for situations in which there is only one Seaside server. A single shared Magma session and object model.</strong>

<br><br>

'Solo' indicates that Seaside is the only user of the database and therefore has the latest data available without any need for an explicit refresh. 

<br><br>

Notes: Manual #commits: etc are protected so that multiple Seaside users do not accidentally overlap the application of their changes to the database.  !

!WAMagmaSolo class methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:11'!
initialize 

 SharedMagmaSessions := Dictionary new.
 SharedMonitors := Dictionary new.! !

WAMagmaSolo subclass: #WAMagmaShared
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Helpers'!

!WAMagmaShared commentStamp: 'kph 12/9/2007 13:29' prior: 0!
<strong>WAMagmaShared, a single shared Magma session and object model</strong>

<br><br>

'Shared' indicates that this seaside server uses a single MagmaSession per database, however it is NOT the only client of the database so the latest data is explicitly refreshed at the beginning of each page render  

!

!WAMagmaShared methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 11:57'!
responseForRequest: aRequest wrap: aBlock

	| result |
	 
	^ [ self abort.  result := aBlock value   "after". result ]
! !

WAMagmaShared subclass: #WAMagmaSharedAuto
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Helpers'!

!WAMagmaSharedAuto commentStamp: 'kph 12/9/2007 13:27' prior: 0!
<strong>WAMagmaSharedAuto, a single shared Magma session and object model, data committing is automatic.</strong>

<br><br>

'Shared' indicates that this seaside server uses a single MagmaSession per database, however it is NOT the only client of the database so the latest data is explicitly refreshed at the beginning of each page render (and a transaction is begun).

<br><br>

'Auto' indicates that application code does not need to commit data into the database. Page rendering is wrapped in a #possiblyCommit: transaction so that following the render the transaction is committed if still in a transaction. 

<br><br>

Notes: Manual #commits: abort the current transaction, commit their own transaction and #begin again.
!

!WAMagmaSharedAuto methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 12:04'!
commit: aBlock

	| result  |
	 
	self mutex critical: 
		[ 
			
			self privateDuring: [ 
				self magmaSession 
						abort ; 
						begin.
				result := aBlock value.
				self magmaSession commitAndBegin.
			]
		].
	
	 ^ result! !

!WAMagmaSharedAuto methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 11:58'!
responseForRequest: aRequest wrap: aBlock

	| result |
	 
	^ [ self possiblyCommit: [ result := aBlock value ] "after". result ]! !

!WAMagmaSolo class methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 04:26'!
configurationComment 

	^  self class comment! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:12'!
abort 

	 self monitor critical: [ super abort  ]
	
	 ! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:12'!
begin 

	 self monitor critical: [ super begin  ]
	
	 ! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:12'!
commit 

	 self monitor critical: [ super commit  ]
	
	 ! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:12'!
commit: aBlock

	| result  |
		
	self monitor critical: 
		[ 	
			result := super commit: aBlock.
		].
	
	 ^ result! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:13'!
commitAndBegin

	 self monitor critical: [ super commitAndBegin ]
	
	 ! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:10'!
monitor

	^ monitor ifNil: [ monitor := self obtainMonitor ]! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 01:58'!
obtainMagmaSession

"super obtainMagmaSession calls updateReadStrategy which assumes the magmaSession is stored 
in the attribute, in this cas it is not, so we fake it temporarily (not thread safe)"

| ms |

ms := SharedMagmaSessions at: (self location) ifAbsent: [ nil ].

((ms isNil or: [ ms isOpen not ]) or: [ ms isConnected not]) 
	ifTrue: [ ms := SharedMagmaSessions at: self location put: super obtainMagmaSession ].

^ ms! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:15'!
obtainMonitor

	^ SharedMonitors at: (self location) ifAbsentPut: [ Monitor new ].

! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:15'!
possiblyCommit: aBlock

	| result  |
		
	self monitor critical: 
		[ 	
			result := super possiblyCommit: aBlock.
		].
	
	 ^ result! !

!WAMagmaSolo methodsFor: 'seaside hooks' stamp: 'kph 12/9/2007 05:28'!
postProcessRequest: aRequest returned: aResponse

 	"occurs after response has been sent to the server"! !

!WAMagmaSolo methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:21'!
tearDown

	"do not discard the magma session, since it is shared"! !

WAMagmaSolo subclass: #WAMagmaSoloAuto
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-Helpers'!

!WAMagmaSoloAuto commentStamp: 'kph 12/9/2007 13:27' prior: 0!
<strong>WAMagmaSoloAuto is specifically designed as the simplest and most transparent of the Magma-Seaside helper options for situations in which there is only one Seaside server. A single shared Magma session and object model. Data commiting is automatic</strong>

<br><br>

'Auto' indicates that application code does not need to commit data into the database. An automatic #commitAndBegin writes data to the database following callback processing, just prior to rendering.

<br><br>

'Solo' indicates that Seaside is the only user of the database and therefore has the latest data available without any need for an explicit refresh. 

<br><br>

Notes: If the user does perform a manual #commit: it is protected so that multiple Seaside users do not accidentally overlap the application of their changes to the database. Since a manual #commit: in this instance is a sub-transaction it would normally be ignored, however WAMagmaSoloAuto performs an #abort prior to any manual #commit: this abort should not revoke any local changes made to the data model, and the manual #commit: will be performed, followed by a #begin in anticipation of the next automatic #commitAndBegin.!

!WAMagmaSoloAuto methodsFor: 'as yet unclassified' stamp: 'kph 12/10/2007 04:17'!
commit: aBlock

	| result  |
	 
	self monitor critical: 
		[ 
			
			self privateDuring: [ 
				self magmaSession 
						abort ; 
						begin.
				result := aBlock value.
				self magmaSession commitAndBegin.
			]
		].
	
	 ^ result! !

!WAMagmaSoloAuto methodsFor: 'as yet unclassified' stamp: 'kph 12/9/2007 05:00'!
updateRoot: aHtmlRoot

	"the start of rendering"
	self commitAndBegin! !

!MagmaLocation class methodsFor: '*magma seasidehelper' stamp: 'kph 12/7/2007 22:24'!
defaultFor: seasideApp

	^ self default! !

!MagmaLocation methodsFor: '*magma seasidehelper' stamp: 'kph 12/7/2007 22:46'!
isValid

	^ true! !

!MagmaLocation methodsFor: '*magma seasidehelper' stamp: 'kph 9/9/2006 03:21'!
newSessionAs: userName

 ^ self newSession connectAs: userName; yourself! !

!MagmaLocation methodsFor: '*magma seasidehelper' stamp: 'kph 12/7/2007 21:41'!
renderContentOn: html
	
	self subclassResponsibility! !

!MagmaLocation methodsFor: '*magma seasidehelper' stamp: 'kph 12/7/2007 21:41'!
renderOn: html

	 
	html div id: 'location'; with: [
		html table: [
			self renderContentOn: html
		]
	].
! !

!MaObjectRepository methodsFor: '*magma seasideHelper-override' stamp: 'kph 6/14/2007 00:01'!
graphBufferAt: oidInteger using: aMaReadStrategy
	"Answers a MaSerializedGraphBuffer."

	| answer rootBuffer |
	answer _ MaSerializedGraphBuffer new: 400.
	rootBuffer _ 
		filer
			appendObject: oidInteger
			into: answer.

	rootBuffer
		appendGraphUsing: (aMaReadStrategy signalBeginningToRead: oidInteger classId: rootBuffer classId)
		into: answer
		currentDepth: 1
		minDepth: aMaReadStrategy minimumDepth
		with: Set new
		filer: filer.
	^ answer! !

WADispatcherPlugin subclass: #WAMagmaDispatcherPlugin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Magma seasideHelper-ControlPanel'!

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/27/2007 02:35'!
controlPanelClass

	^ WAMagmaControlPanel! !

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/27/2007 03:49'!
execute 

	self requestor call:  (self controlPanelClass forApplication: self target)! !

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/18/2007 11:45'!
isActive 
	^ target isApplication and: 
		[ target configuration allAttributes anySatisfy: [ :c | c group = #magma]  ]! !

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/11/2007 17:01'!
label

	^ 'Magma'! !

!WAMagmaDispatcherPlugin methodsFor: 'as yet unclassified' stamp: 'kph 7/11/2007 16:57'!
priority

	^ 400! !

!WASession methodsFor: '*magma seasidehelper' stamp: 'kph 12/9/2007 06:06'!
magma
	^ self helpers at: #magma ifAbsent: [ self error: 'WAMagmaConfiguration not selected in application configuration' ]! !

!WASession methodsFor: '*magma seasidehelper' stamp: 'kph 12/9/2007 06:06'!
magma2
	^ self helpers at: #magma2 ifAbsent: [ self error: 'WAMagmaConfiguration2nd not selected in application configuration' ]! !

!MagmaCollectionReader methodsFor: '*Magma seasideHelper' stamp: 'kph 10/28/2007 21:35'!
where: aBlock orSelect: bBlock
	 
	^ self where: aBlock! !

!MagmaSession methodsFor: '*magma seasidehelper' stamp: 'kph 12/9/2007 11:02'!
wait

"dummy method"

	^ true! !
WAMagmaControlPanel initialize!
WAMagmaSolo initialize!
